# coding: utf-8

"""
    Kubeflow Training SDK

    Python SDK for Kubeflow Training  # noqa: E501

    The version of the OpenAPI document: v1.6.0rc0
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import unittest
import datetime

from kubeflow.training.models import *
from kubeflow.training.models.v1_job_condition import V1JobCondition  # noqa: E501
from kubeflow.training.rest import ApiException

class TestV1JobCondition(unittest.TestCase):
    """V1JobCondition unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional):
        """Test V1JobCondition
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # model = kubeflow.training.models.v1_job_condition.V1JobCondition()  # noqa: E501
        if include_optional :
            return V1JobCondition(
                last_transition_time = None, 
                last_update_time = None, 
                message = '0', 
                reason = '0', 
                status = '0', 
                type = '0'
            )
        else :
            return V1JobCondition(
                status = '0',
                type = '0',
        )

    def testV1JobCondition(self):
        """Test V1JobCondition"""
        inst_req_only = self.make_instance(include_optional=False)
        inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == '__main__':
    unittest.main()
